#!/bin/sh

# copied from powerlaunch

source /etc/asui-svc

DAEMON=/usr/bin/advanced-systemui
#PIDFILE=/var/run/advanced-systemui.pid
INITFILE=/etc/init.d/advanced-systemui-early

PATH=/sbin:/bin:/usr/sbin:/usr/bin
DSMETOOL=/usr/sbin/dsmetool
DSMETOOL_PARAMETERS="-G root -U root -n -1 -f "
STOP_PARAMETERS="-k "

# abort if no executable exists
test -x $DAEMON || exit 0

# from osso-systemui-early
if [ -e /home/user/.icons ]; then
	rm -f /tmp/.icons
	ln -s /home/user/.icons /tmp/.icons
fi

set -e

source /etc/osso-af-init/af-defines.sh

asui_start()
{
	if ! pidof $DAEMON > /dev/null; then
		[ "$1" = "1" ] && printf "Starting Advanced SystemUI..."
		$DSMETOOL $DSMETOOL_PARAMETERS $DAEMON
		printf ".\n"
	fi
}
asui_stop()
{
	$DSMETOOL $STOP_PARAMETERS $DAEMON
	sleep 2
	if pidof $DAEMON > /dev/null; then
		$DSMETOOL -S 9 $STOP_PARAMETERS $DAEMON
		sleep 1
	fi
}

case "$1" in
start)
	# from osso-systemui-early
	LOCK=`dbus-send --system --type=method_call --print-reply --dest=com.nokia.mce /com/nokia/mce/request com.nokia.mce.request.get_devicelock_mode|grep string|cut -d'"' -f2`
	if grep -q '\(charger\|rtc_alarm\)' /proc/bootreason; then
		killall -9 fb-chaimage 2>/dev/null || true
		LOCK="locked"
	fi
	if [ "$LOCK" != "unlocked" ] || [ -f "/home/user/first-boot-flag" ]; then
		# SystemUI can't be running alongside ASUI since we need to steal its dbus name
		if pidof systemui > /dev/null; then
			printf "* systemui-early is running - stopping it...\n"
			/etc/init.d/osso-systemui-early stop || exit $?
		fi

		asui_start 1
	fi
	;;

stop)
	;;

restart|force-reload)
	printf "Restarting Advanced SystemUI..."
	asui_stop
	asui_start 0
	;;

*)
	printf "Usage: $INITFILE {start|stop|restart|force-reload}\n" >&2
	exit 1
	;;
esac

exit 0
